from matplotlib.figure import Figure

from .csvruntable import CsvRunTable
from .dataset import Dataset
from .httprequests import HttpRequests
from .imagerequest import ImageRequest
from .jsondump import JsonDump
from .proposal import Proposal


class ScicatClient:

    def __init__(self, proposal_id: int, source_folder: str):
        self._proposal_id = proposal_id
        self._source_folder = source_folder
        self._save_response = False
        self._json_dump_path = None

    @property
    def proposal(self):
        return Proposal(self._proposal_id).proposal

    def push_dataset(self, run_name: str, run_id: str, scientific_metadata: dict, datatype='raw') -> None:
        dataset = Dataset(self._proposal_id, run_id)
        dataset.create_metadata(run_name, scientific_metadata, datatype, self._source_folder)
        if self._save_response:
            JsonDump(self._json_dump_path).dump_to_disk(run_id, dataset.metadata)
        response = HttpRequests().push_dataset(dataset)
        if response.ok:
            print(f"Dataset {run_id} successfully ingested.")
        else:
            raise Exception(f"{response.text}")

    def update_dataset(self, run_id: str, scientific_metadata: dict) -> None:
        dataset = Dataset(self._proposal_id, run_id)
        dataset.update_metadata(scientific_metadata)
        if self._save_response:
            JsonDump(self._json_dump_path).dump_to_disk(run_id, dataset.metadata)
        response = HttpRequests().update_dataset(dataset)
        if response.ok:
            print(f"Dataset {run_id} successfully updated.")
        else:
            raise Exception(f"{response.text}")

    def get_dataset(self, run_id: str) -> dict:
        return HttpRequests().get_dataset(self._build_pid(run_id))

    def get_all_datasets(self) -> list:
        return HttpRequests().get_all_datasets(self._proposal_id)

    def create_csv_run_table(self, path: str, collapse_unit_columns: bool = False) -> None:
        CsvRunTable(self.get_all_datasets(), collapse_unit_columns).dump(path)

    def add_image_to_dataset(self, run_id: str, image: Figure, caption: str) -> None:
        dataset = self.get_dataset(run_id)
        image_request = ImageRequest(dataset, image, caption).create()
        if self._save_response:
            JsonDump(self._json_dump_path).dump_to_disk(run_id, image_request)
        response = HttpRequests().add_image_to_dataset(self._build_pid(run_id), image_request)
        if response.ok:
            print(f"Added image to dataset {run_id} successfully.")
        else:
            raise Exception(f"{response.text}")

    def activate_json_dump(self, json_dump_path: str) -> None:
        self._save_response = True
        self._json_dump_path = json_dump_path

    def deactivate_json_dump(self) -> None:
        self._save_response = False
        self._json_dump_path = None

    def _build_pid(self, run_id: str) -> str:
        return f"{self._proposal_id}/{run_id}"
