import math
from functools import reduce
from pathlib import Path

from pandas import DataFrame, Series
from pandas.io.json._normalize import nested_to_record


class CsvRunTable:
    TOP_LEVEL_KEYS = ['datasetName', 'pid', 'scientificMetadata']

    def __init__(self, datasets: list, collapse_unit_columns: bool):
        self._datasets = datasets
        self._collapse_unit_columns = collapse_unit_columns

    def dump(self, path: str) -> None:
        if self._collapse_unit_columns:
            run_table = self._create_run_table_collapsed()
        else:
            run_table = self._create_run_table()
        filepath = Path(path)
        filepath.parent.mkdir(parents=True, exist_ok=True)
        run_table.sort_index(axis=1, inplace=True)
        run_table.to_csv(filepath)

    def _create_run_table(self) -> DataFrame:
        data_frames = [self._create_row(each) for each in self._datasets]
        run_table = reduce(DataFrame.combine_first, data_frames,
                           DataFrame(columns=self._create_header(self._datasets)))
        return run_table

    def _create_run_table_collapsed(self) -> DataFrame:
        run_table = self._create_run_table()
        unit_columns = [col for col in run_table.columns if col.endswith(' unit')]
        for each in unit_columns:
            if CsvRunTable._is_unique(run_table[each]):
                a = CsvRunTable._get_unique(run_table[each])
                b = each.replace(' unit', ' value')
                c = f"{b[:-5]}[{a}]"
                run_table.rename(columns={b: c}, inplace=True)
                run_table.drop(columns=[each], inplace=True)

        unit_si_columns = [col for col in run_table.columns if col.endswith(' unitSI')]
        for each in unit_si_columns:
            if CsvRunTable._is_unique(run_table[each]):
                a = CsvRunTable._get_unique(run_table[each])
                b = each.replace(' unitSI', ' valueSI')
                c = f"{b[:-7]}[{a}]"
                run_table.rename(columns={b: c}, inplace=True)
                run_table.drop(columns=[each], inplace=True)
        return run_table

    @staticmethod
    def _remove_prefix(text, prefix: str):
        if isinstance(text, str):
            return text[text.startswith(prefix) and len(prefix):]
        return str(text)

    @staticmethod
    def _create_header(datasets: list):
        header = []
        for run in datasets:
            run_header = nested_to_record(run, sep=' ')
            for key in run_header.keys():
                if any(x in key for x in CsvRunTable.TOP_LEVEL_KEYS):
                    header.append(CsvRunTable._remove_prefix(key, 'scientificMetadata '))
        return list(dict.fromkeys(header))

    @staticmethod
    def _create_row(dataset):
        header = []
        values = []
        unnested_dataset = nested_to_record(dataset, sep=' ')
        for key in unnested_dataset.keys():
            if any(x in key for x in CsvRunTable.TOP_LEVEL_KEYS):
                header.append(CsvRunTable._remove_prefix(key, 'scientificMetadata '))
                values.append(CsvRunTable._remove_prefix(unnested_dataset[key], '/'))
        row = dict(zip(header, values))
        return DataFrame(row, index=[row['pid']])

    @staticmethod
    def _is_unique(s: Series) -> bool:
        a = set(s.to_list())
        if len(a) == 1:
            return True
        if len(a) == 2:
            for each in a:
                if isinstance(each, float) and math.isnan(each):
                    return True
        return False

    @staticmethod
    def _get_unique(s: Series) -> str:
        return {x for x in (set(s.to_list())) if x == x}.pop()
